@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Linguist of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "clangtoolastreader.h"
 #include "filesignificancecheck.h"
@@ -790,9 +765,9 @@ bool LupdateVisitor::VisitNamedDecl(clang::NamedDecl *namedDeclaration)
     if (!LupdatePrivate::isFileSignificant(fullLocation.getFileEntry()->getName().str()))
         return true;
 
-    qCDebug(lcClang) << "NamedDecl Name:   " << namedDeclaration->getQualifiedNameAsString();
-    qCDebug(lcClang) << "NamedDecl source: " << namedDeclaration->getSourceRange().printToString(
-        m_context->getSourceManager());
+    qCDebug(lcClang) << "NamedDecl Name:   " << QString::fromStdString(namedDeclaration->getQualifiedNameAsString());
+    qCDebug(lcClang) << "NamedDecl source: " << QString::fromStdString(namedDeclaration->getSourceRange().printToString(
+        m_context->getSourceManager()));
     // Checks if there is a macro located within the range of this NamedDeclaration
     // in order to find a context for the macro
     findContextForTranslationStoresFromPP(namedDeclaration);
@@ -820,15 +795,15 @@ void LupdateVisitor::findContextForTranslationStoresFromPP(clang::NamedDecl *nam
 
             store.contextRetrieved = LupdatePrivate::contextForNoopMacro(namedDeclaration, sm);
             qCDebug(lcClang) << "------------------------------------------NOOP Macro in range ---";
-            qCDebug(lcClang) << "Range " << namedDeclaration->getSourceRange().printToString(sm);
-            qCDebug(lcClang) << "Point " << sourceLoc.printToString(sm);
+            qCDebug(lcClang) << "Range " << QString::fromStdString(namedDeclaration->getSourceRange().printToString(sm));
+            qCDebug(lcClang) << "Point " << QString::fromStdString(sourceLoc.printToString(sm));
             qCDebug(lcClang) << "=========== Visit Named Declaration =============================";
             qCDebug(lcClang) << " Declaration Location    " <<
-                namedDeclaration->getSourceRange().printToString(sm);
+                QString::fromStdString(namedDeclaration->getSourceRange().printToString(sm));
             qCDebug(lcClang) << " Macro       Location                                 "
-                << sourceLoc.printToString(sm);
+                << QString::fromStdString(sourceLoc.printToString(sm));
             qCDebug(lcClang) << " Context namedDeclaration->getQualifiedNameAsString() "
-                << namedDeclaration->getQualifiedNameAsString();
+                << QString::fromStdString(namedDeclaration->getQualifiedNameAsString());
             qCDebug(lcClang) << " Context LupdatePrivate::contextForNoopMacro          "
                 << store.contextRetrieved;
             qCDebug(lcClang) << " Context Retrieved       " << store.contextRetrieved;
@@ -845,13 +820,13 @@ void LupdateVisitor::findContextForTranslationStoresFromPP(clang::NamedDecl *nam
             store.contextRetrieved = QString::fromStdString(
                 namedDeclaration->getQualifiedNameAsString());
             qCDebug(lcClang) << "------------------------------------------DECL Macro in range ---";
-            qCDebug(lcClang) << "Range " << namedDeclaration->getSourceRange().printToString(sm);
-            qCDebug(lcClang) << "Point " << sourceLoc.printToString(sm);
+            qCDebug(lcClang) << "Range " << QString::fromStdString(namedDeclaration->getSourceRange().printToString(sm));
+            qCDebug(lcClang) << "Point " << QString::fromStdString(sourceLoc.printToString(sm));
             qCDebug(lcClang) << "=========== Visit Named Declaration =============================";
             qCDebug(lcClang) << " Declaration Location    " <<
-                namedDeclaration->getSourceRange().printToString(sm);
+                QString::fromStdString(namedDeclaration->getSourceRange().printToString(sm));
             qCDebug(lcClang) << " Macro       Location                                 "
-                << sourceLoc.printToString(sm);
+                << QString::fromStdString(sourceLoc.printToString(sm));
             qCDebug(lcClang) << " Context namedDeclaration->getQualifiedNameAsString() "
                 << store.contextRetrieved;
             qCDebug(lcClang) << " Context Retrieved       " << store.contextRetrieved;
