@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "helpprojectwriter.h"
 
@@ -149,13 +124,12 @@ void HelpProjectWriter::readSelectors(SubProject &subproject, const QStringList
     typeHash["variable"] = Node::Variable;
     typeHash["group"] = Node::Group;
     typeHash["module"] = Node::Module;
-    typeHash["jsmodule"] = Node::JsModule;
     typeHash["qmlmodule"] = Node::QmlModule;
-    typeHash["qmlproperty"] = Node::JsProperty;
-    typeHash["jsproperty"] = Node::QmlProperty;
+    typeHash["qmlproperty"] = Node::QmlProperty;
     typeHash["qmlclass"] = Node::QmlType; // Legacy alias for 'qmltype'
     typeHash["qmltype"] = Node::QmlType;
-    typeHash["qmlbasictype"] = Node::QmlBasicType;
+    typeHash["qmlbasictype"] = Node::QmlValueType; // Legacy alias for 'qmlvaluetype'
+    typeHash["qmlvaluetype"] = Node::QmlValueType;
 
     for (const QString &selector : selectors) {
         QStringList pieces = selector.split(QLatin1Char(':'));
@@ -170,11 +144,10 @@ void HelpProjectWriter::readSelectors(SubProject &subproject, const QStringList
         subproject.m_selectors << typeHash.value(typeName);
         if (!pieces.isEmpty()) {
             pieces = pieces[0].split(QLatin1Char(','));
-            for (const auto &piece : qAsConst(pieces)) {
+            for (const auto &piece : std::as_const(pieces)) {
                 if (typeHash[typeName] == Node::Group
                     || typeHash[typeName] == Node::Module
-                    || typeHash[typeName] == Node::QmlModule
-                    || typeHash[typeName] == Node::JsModule) {
+                    || typeHash[typeName] == Node::QmlModule) {
                     subproject.m_groups << piece.toLower();
                 }
             }
@@ -200,7 +173,7 @@ Keyword HelpProjectWriter::keywordDetails(const Node *node) const
                 ? node->parent()->name()+"::"+node->name()
                 : node->name();
         return Keyword(name, id, ref);
-    } else if (node->isQmlType() || node->isQmlBasicType()) {
+    } else if (node->isQmlType()) {
         const QString &name = node->name();
         QString moduleName = node->logicalModuleName();
         QStringList ids("QML." + name);
@@ -211,8 +184,11 @@ Keyword HelpProjectWriter::keywordDetails(const Node *node) const
             ids << "QML." + moduleName + majorVersion + "." + name;
         }
         return Keyword(name, ids, ref);
-    } else if (node->isJsType() || node->isJsBasicType()) {
-        return Keyword(node->name(), "JS." + node->name(), ref);
+    } else if (node->isQmlModule()) {
+        const QLatin1Char delim('.');
+        QStringList parts = node->logicalModuleName().split(delim) << "QML";
+        std::reverse(parts.begin(), parts.end());
+        return Keyword(node->logicalModuleName(), parts.join(delim), ref);
     } else if (node->isTextPageNode()) {
         const auto *pageNode = static_cast<const PageNode *>(node);
         return Keyword(pageNode->fullTitle(), pageNode->fullTitle(), ref);
@@ -241,7 +217,7 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
     // Only add nodes to the set for each subproject if they match a selector.
     // Those that match will be listed in the table of contents.
 
-    for (int i = 0; i < project.m_subprojects.length(); i++) {
+    for (int i = 0; i < project.m_subprojects.size(); i++) {
         SubProject subproject = project.m_subprojects[i];
         // No selectors: accept all nodes.
         if (subproject.m_selectors.isEmpty()) {
@@ -279,9 +255,7 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
         project.m_keywords.append(keywordDetails(node));
         break;
     case Node::QmlType:
-    case Node::QmlBasicType:
-    case Node::JsType:
-    case Node::JsBasicType:
+    case Node::QmlValueType:
         if (node->doc().hasKeywords()) {
             const auto keywords = node->doc().keywords();
             for (const Atom *keyword : keywords) {
@@ -326,8 +300,7 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
 
     case Node::Group:
     case Node::Module:
-    case Node::QmlModule:
-    case Node::JsModule: {
+    case Node::QmlModule: {
         const auto *cn = static_cast<const CollectionNode *>(node);
         if (!cn->fullTitle().isEmpty()) {
             if (cn->doc().hasKeywords()) {
@@ -349,7 +322,6 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
 
     case Node::Property:
     case Node::QmlProperty:
-    case Node::JsProperty:
         project.m_keywords.append(keywordDetails(node));
         break;
 
@@ -357,13 +329,11 @@ bool HelpProjectWriter::generateSection(HelpProject &project, QXmlStreamWriter &
         const auto *funcNode = static_cast<const FunctionNode *>(node);
 
         /*
-          QML and JS methods, signals, and signal handlers used to be node types,
+          QML methods, signals, and signal handlers used to be node types,
           but now they are Function nodes with a Metaness value that specifies
-          what kind of function they are, QmlSignal, JsSignal, QmlMethod, etc. It
-          suffices at this point to test whether the node is of the QML or JS Genus,
-          because we already know it is NodeType::Function.
-         */
-        if (funcNode->isQmlNode() || funcNode->isJsNode()) {
+          what kind of function they are, QmlSignal, QmlMethod, etc.
+        */
+        if (funcNode->isQmlNode()) {
             project.m_keywords.append(keywordDetails(node));
             break;
         }
@@ -475,7 +445,7 @@ void HelpProjectWriter::generateSections(HelpProject &project, QXmlStreamWriter
                     childSet << child;
             }
         }
-        for (const auto *child : qAsConst(childSet))
+        for (const auto *child : std::as_const(childSet))
             generateSections(project, writer, child);
     }
 }
@@ -492,7 +462,7 @@ void HelpProjectWriter::writeHashFile(QFile &file)
     hash.addData(&file);
 
     QFile hashFile(file.fileName() + ".sha1");
-    if (!hashFile.open(QFile::WriteOnly | QFile::Text))
+    if (!hashFile.open(QFile::WriteOnly))
         return;
 
     hashFile.write(hash.result().toHex());
@@ -513,9 +483,6 @@ void HelpProjectWriter::writeSection(QXmlStreamWriter &writer, const QString &pa
  */
 void HelpProjectWriter::addMembers(HelpProject &project, QXmlStreamWriter &writer, const Node *node)
 {
-    if (node->isQmlBasicType() || node->isJsBasicType())
-        return;
-
     QString href = m_gen->fullDocumentLocation(node, false);
     href = href.left(href.size() - 5);
     if (href.isEmpty())
@@ -526,10 +493,9 @@ void HelpProjectWriter::addMembers(HelpProject &project, QXmlStreamWriter &write
         derivedClass = !(static_cast<const ClassNode *>(node)->baseClasses().isEmpty());
 
     // Do not generate a 'List of all members' for namespaces or header files,
-    // but always generate it for derived classes and QML classes
-    if (!node->isNamespace() && !node->isHeader()
-        && (derivedClass || node->isQmlType() || node->isJsType()
-            || !project.m_memberStatus[node].isEmpty())) {
+    // but always generate it for derived classes and QML types (but not QML value types)
+    if (!node->isNamespace() && !node->isHeader() && !node->isQmlBasicType()
+        && (derivedClass || node->isQmlType() || !project.m_memberStatus[node].isEmpty())) {
         QString membersPath = href + QStringLiteral("-members.html");
         writeSection(writer, membersPath, QStringLiteral("List of all members"));
     }
@@ -550,9 +516,7 @@ void HelpProjectWriter::writeNode(HelpProject &project, QXmlStreamWriter &writer
     case Node::Struct:
     case Node::Union:
     case Node::QmlType:
-    case Node::JsType:
-    case Node::QmlBasicType:
-    case Node::JsBasicType: {
+    case Node::QmlValueType: {
         QString typeStr = m_gen->typeString(node);
         if (!typeStr.isEmpty())
             typeStr[0] = typeStr[0].toTitleCase();
@@ -578,7 +542,6 @@ void HelpProjectWriter::writeNode(HelpProject &project, QXmlStreamWriter &writer
     case Node::Page:
     case Node::Group:
     case Node::Module:
-    case Node::JsModule:
     case Node::QmlModule: {
         writer.writeStartElement("section");
         writer.writeAttribute("ref", href);
@@ -611,7 +574,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
     project.m_keywords.clear();
 
     QFile file(m_outputDir + QDir::separator() + project.m_fileName);
-    if (!file.open(QFile::WriteOnly | QFile::Text))
+    if (!file.open(QFile::WriteOnly))
         return;
 
     QXmlStreamWriter writer(&file);
@@ -635,7 +598,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
         writer.writeAttribute("name", it.key());
         QStringList sortedAttributes = it.value().values();
         sortedAttributes.sort();
-        for (const auto &filter : qAsConst(sortedAttributes))
+        for (const auto &filter : std::as_const(sortedAttributes))
             writer.writeTextElement("filterAttribute", filter);
         writer.writeEndElement(); // customFilter
     }
@@ -646,7 +609,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
     // Write filterAttribute elements.
     QStringList sortedFilterAttributes = project.m_filterAttributes.values();
     sortedFilterAttributes.sort();
-    for (const auto &filterName : qAsConst(sortedFilterAttributes))
+    for (const auto &filterName : std::as_const(sortedFilterAttributes))
         writer.writeTextElement("filterAttribute", filterName);
 
     writer.writeStartElement("toc");
@@ -666,7 +629,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
 
     generateSections(project, writer, rootNode);
 
-    for (int i = 0; i < project.m_subprojects.length(); i++) {
+    for (int i = 0; i < project.m_subprojects.size(); i++) {
         SubProject subproject = project.m_subprojects[i];
 
         if (subproject.m_type == QLatin1String("manual")) {
@@ -729,14 +692,14 @@ void HelpProjectWriter::generateProject(HelpProject &project)
             if (subproject.m_sortPages) {
                 QStringList titles = subproject.m_nodes.keys();
                 titles.sort();
-                for (const auto &title : qAsConst(titles)) {
+                for (const auto &title : std::as_const(titles)) {
                     writeNode(project, writer, subproject.m_nodes[title]);
                 }
             } else {
                 // Find a contents node and navigate from there, using the NextLink values.
                 QSet<QString> visited;
                 bool contentsFound = false;
-                for (const auto *node : qAsConst(subproject.m_nodes)) {
+                for (const auto *node : std::as_const(subproject.m_nodes)) {
                     QString nextTitle = node->links().value(Node::NextLink).first;
                     if (!nextTitle.isEmpty()
                         && node->links().value(Node::ContentsLink).first.isEmpty()) {
@@ -764,7 +727,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
 
                     std::sort(subnodes.begin(), subnodes.end(), Node::nodeNameLessThan);
 
-                    for (const auto *node : qAsConst(subnodes))
+                    for (const auto *node : std::as_const(subnodes))
                         writeNode(project, writer, node);
                 }
             }
@@ -781,8 +744,8 @@ void HelpProjectWriter::generateProject(HelpProject &project)
 
     writer.writeStartElement("keywords");
     std::sort(project.m_keywords.begin(), project.m_keywords.end());
-    for (const auto &k : qAsConst(project.m_keywords)) {
-        for (const auto &id : qAsConst(k.m_ids)) {
+    for (const auto &k : std::as_const(project.m_keywords)) {
+        for (const auto &id : std::as_const(k.m_ids)) {
             writer.writeStartElement("keyword");
             writer.writeAttribute("name", k.m_name);
             writer.writeAttribute("id", id);
@@ -802,7 +765,7 @@ void HelpProjectWriter::generateProject(HelpProject &project)
     files.unite(project.m_extraFiles);
     QStringList sortedFiles = files.values();
     sortedFiles.sort();
-    for (const auto &usedFile : qAsConst(sortedFiles)) {
+    for (const auto &usedFile : std::as_const(sortedFiles)) {
         if (!usedFile.isEmpty())
             writer.writeTextElement("file", usedFile);
     }
