@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #include <QProcess>
 #include <QTemporaryDir>
 #include <QDirIterator>
@@ -52,6 +27,7 @@ private slots:
     void webXmlFromCpp();
     void webXmlFromQml();
     void webXmlFromCppBug80259();
+    void illformated_documentation_caused_qtbug112641();
 
     // DocBook generator
     void docBookFromQDocFile();
@@ -59,6 +35,8 @@ private slots:
     void docBookFromQml();
 
     // Output format independent tests
+    void autoNavigation();
+    void tocBreadcrumbs();
     void examplesManifestXmlAndQhp();
     void ignoresinceVariable();
     void templateParameters();
@@ -66,6 +44,7 @@ private slots:
     void dontDocument();
     void inheritedQmlPropertyGroups();
     void crossModuleLinking();
+    void indexLinking();
     void includeFromExampleDirs();
     void singleExec();
     void preparePhase();
@@ -73,11 +52,18 @@ private slots:
     void noAutoList();
     void nestedMacro();
     void headerFile();
+    void usingDirective();
+    void properties();
+    void testTagFile();
+    void testGlobalFunctions();
+    void proxyPage();
+    void tableAfterValue();
 
 private:
     QScopedPointer<QTemporaryDir> m_outputDir;
     QString m_qdoc;
     QDir m_expectedDir;
+    QString m_extraParams;
     bool m_regen = false;
 
     void runQDocProcess(const QStringList &arguments);
@@ -90,10 +76,20 @@ private:
 void tst_generatedOutput::initTestCase()
 {
     // Build the path to the QDoc binary the same way moc tests do for moc.
-    const auto binpath = QLibraryInfo::location(QLibraryInfo::BinariesPath);
+    const auto binpath = QLibraryInfo::path(QLibraryInfo::BinariesPath);
     const auto extension = QSysInfo::productType() == "windows" ? ".exe" : "";
     m_qdoc = binpath + QLatin1String("/qdoc") + extension;
-    m_expectedDir.setPath(QFINDTESTDATA(".") + QLatin1String("/expected_output"));
+    m_expectedDir.setPath(QFINDTESTDATA("expected_output"));
+
+    // Resolve the path to the file containing extra parameters
+    m_extraParams = QFileInfo(QTest::currentAppName()).dir().filePath("qdocincludepaths.inc");
+    if (!QFileInfo::exists(m_extraParams)) {
+        qWarning().nospace() << QStringLiteral("Cannot locate")
+                             << qUtf8Printable(m_extraParams);
+        m_extraParams.clear();
+    } else {
+        m_extraParams.insert(0, '@');
+    }
 }
 
 void tst_generatedOutput::init()
@@ -111,22 +107,23 @@ void tst_generatedOutput::runQDocProcess(const QStringList &arguments)
     QProcess qdocProcess;
     qdocProcess.setProgram(m_qdoc);
     qdocProcess.setArguments(arguments);
+
+    auto failQDoc = [&](QProcess::ProcessError) {
+        QFAIL(qPrintable(QStringLiteral("Running qdoc failed with exit code %1: %2")
+                .arg(qdocProcess.exitCode()).arg(qdocProcess.errorString())));
+    };
+    QObject::connect(&qdocProcess, &QProcess::errorOccurred, failQDoc);
+
     qdocProcess.start();
     qdocProcess.waitForFinished();
-
     if (qdocProcess.exitCode() == 0)
         return;
 
-    QString output = qdocProcess.readAllStandardOutput();
     QString errors = qdocProcess.readAllStandardError();
-
-    qInfo() << "QDoc exited with exit code" << qdocProcess.exitCode();
-    if (output.size() > 0)
-        qInfo() << "Received output:\n" << output;
-    if (errors.size() > 0)
-        qInfo() << "Received errors:\n" << errors;
-
-    QFAIL("Running QDoc failed. See output above.");
+    if (!errors.isEmpty())
+        qInfo().nospace() << "Received errors:\n" << qUtf8Printable(errors);
+    if (!QTest::currentTestFailed())
+        failQDoc(QProcess::UnknownError);
 }
 
 void tst_generatedOutput::compareLineByLine(const QStringList &expectedFiles)
@@ -177,15 +174,14 @@ void tst_generatedOutput::testAndCompare(const char *input, const char *outNames
 
     if (m_regen) {
         QVERIFY(m_expectedDir.mkpath(m_expectedDir.path()));
-        for (const auto &file : qAsConst(expectedOuts)) {
+        for (const auto &file : std::as_const(expectedOuts)) {
             QFileInfo fileInfo(m_expectedDir.filePath(file));
             fileInfo.dir().remove(fileInfo.fileName()); // Allowed to fail
             QVERIFY(m_expectedDir.mkpath(fileInfo.dir().path()));
-            QVERIFY(QFile::copy(m_outputDir->filePath(file),
-                                fileInfo.filePath()));
+            QVERIFY2(QFile::copy(m_outputDir->filePath(file), fileInfo.filePath()),
+                     qPrintable(QStringLiteral("Failed to copy '%1'").arg(file)));
         }
         QSKIP("Regenerated expected output only.");
-        return;
     }
 
     compareLineByLine(expectedOuts);
@@ -209,7 +205,10 @@ void tst_generatedOutput::htmlFromQDocFile()
 {
     testAndCompare("testdata/configs/test.qdocconf",
                    "qdoctests-qdocfileoutput.html "
-                   "qdoctests-qdocfileoutput-linking.html");
+                   "qdoctests-qdocfileoutput-linking.html "
+                   "qdoctests-qdocmanuallikefileoutput.html "
+                   "qdoctests-qdocfileoutput-exhaustive.html "
+                   "toc.html");
 }
 
 void tst_generatedOutput::htmlFromCpp()
@@ -218,23 +217,34 @@ void tst_generatedOutput::htmlFromCpp()
                    "testcpp-module.html "
                    "testqdoc-test.html "
                    "testqdoc-test-members.html "
+                   "testqdoc-test-obsolete.html "
                    "testqdoc-testderived.html "
                    "testqdoc-testderived-members.html "
+                   "testqdoc-testderived-obsolete.html "
+                   "obsolete-classes.html "
+                   "autolinking.html "
+                   "cpptypes.html "
                    "testqdoc.html");
 }
 
 void tst_generatedOutput::htmlFromQml()
 {
     testAndCompare("testdata/configs/testqml.qdocconf",
+                   "qmlmodules.html "
                    "test-componentset-example.html "
                    "test-cmaketest-example.html "
                    "uicomponents-qmlmodule.html "
                    "qdoc-test-qmlmodule.html "
+                   "test-nover-qmlmodule.html "
                    "qml-qdoc-test-abstractparent.html "
                    "qml-qdoc-test-child.html "
+                   "qml-qdoc-test-yetanotherchild.html "
                    "qml-qdoc-test-doctest.html "
                    "qml-qdoc-test-type-members.html "
+                   "qml-qdoc-test-type-obsolete.html "
                    "qml-qdoc-test-type.html "
+                   "qml-test-nover-typenoversion.html "
+                   "qml-test-nover-typenoversion-members.html "
                    "qml-uicomponents-progressbar.html "
                    "qml-uicomponents-switch.html "
                    "qml-uicomponents-tabwidget.html "
@@ -254,7 +264,9 @@ void tst_generatedOutput::webXmlFromQDocFile()
 {
     testAndCompare("testdata/configs/webxml_test.qdocconf",
                    "html/qdoctests-qdocfileoutput.webxml "
-                   "html/qdoctests-qdocfileoutput-linking.webxml");
+                   "html/qdoctests-qdocmanuallikefileoutput.webxml "
+                   "html/qdoctests-qdocfileoutput-linking.webxml "
+                   "html/qdoctests-qdocfileoutput-exhaustive.webxml");
 }
 
 void tst_generatedOutput::webXmlFromCpp()
@@ -269,6 +281,7 @@ void tst_generatedOutput::webXmlFromQml()
 {
     testAndCompare("testdata/configs/webxml_testqml.qdocconf",
                    "html/test-componentset-example.webxml "
+                   "html/test-nover-qmlmodule.webxml "
                    "html/uicomponents-qmlmodule.webxml");
 }
 
@@ -281,11 +294,31 @@ void tst_generatedOutput::webXmlFromCppBug80259()
                    "html/index.webxml");
 }
 
+void tst_generatedOutput::illformated_documentation_caused_qtbug112641()
+{
+    testAndCompare("testdata/illformatted_documentation/illformatted_documentation.qdocconf",
+                   "html/illformatted-examples.webxml "
+                   "html/illformatteddocumentation-someexample-example.webxml "
+                   "html/illformatteddocumentation.index");
+}
+
+void tst_generatedOutput::tableAfterValue()
+{
+    testAndCompare("testdata/tables/table-after-value.qdocconf",
+                   "tableaftervalue/tableaftervalue-members.html "
+                   "tableaftervalue/tableaftervalue.html "
+                   "tableaftervalue/tableaftervalue.index "
+                   "tableaftervalue/tableaftervalue.webxml "
+                   "tableaftervalue/tableaftervalue.xml");
+}
+
 void tst_generatedOutput::docBookFromQDocFile()
 {
     testAndCompare("testdata/configs/docbook_test.qdocconf",
                    "docbook/qdoctests-qdocfileoutput.xml "
-                   "docbook/qdoctests-qdocfileoutput-linking.xml");
+                   "docbook/qdoctests-qdocmanuallikefileoutput.xml "
+                   "docbook/qdoctests-qdocfileoutput-linking.xml "
+                   "docbook/qdoctests-qdocfileoutput-exhaustive.xml");
 }
 
 void tst_generatedOutput::docBookFromCpp()
@@ -294,6 +327,7 @@ void tst_generatedOutput::docBookFromCpp()
                    "docbook/testcpp-module.xml "
                    "docbook/testqdoc-test.xml "
                    "docbook/testqdoc-testderived.xml "
+                   "docbook/cpptypes.xml "
                    "docbook/testqdoc.xml");
 }
 
@@ -303,21 +337,44 @@ void tst_generatedOutput::docBookFromQml()
                    "docbook/test-componentset-example.xml "
                    "docbook/uicomponents-qmlmodule.xml "
                    "docbook/qdoc-test-qmlmodule.xml "
+                   "docbook/test-nover-qmlmodule.xml "
                    "docbook/qml-qdoc-test-abstractparent.xml "
                    "docbook/qml-qdoc-test-child.xml "
+                   "docbook/qml-qdoc-test-yetanotherchild.xml "
                    "docbook/qml-qdoc-test-doctest.xml "
                    "docbook/qml-qdoc-test-type.xml "
+                   "docbook/qml-test-nover-typenoversion.xml "
                    "docbook/qml-uicomponents-progressbar.xml "
                    "docbook/qml-uicomponents-switch.xml "
                    "docbook/qml-uicomponents-tabwidget.xml "
                    "docbook/qml-int.xml");
 }
 
+void tst_generatedOutput::autoNavigation()
+{
+    // Same expected files as htmlFromQdocFile, but with auto-generated navigation links
+    testAndCompare("testdata/configs/tocnavigation.qdocconf",
+                   "qdoctests-qdocfileoutput.html "
+                   "qdoctests-qdocfileoutput-linking.html "
+                   "qdoctests-qdocfileoutput-exhaustive.html "
+                   "toc.html");
+}
+
+void tst_generatedOutput::tocBreadcrumbs()
+{
+    testAndCompare("testdata/configs/tocbreadcrumbs.qdocconf",
+                   "tocbreadcrumbs/qdoctests-qdocfileoutput.html "
+                   "tocbreadcrumbs/qdoctests-qdocfileoutput-linking.html "
+                   "tocbreadcrumbs/qdoctests-qdocfileoutput-exhaustive.html "
+                   "tocbreadcrumbs/toc-test.html");
+}
+
+
 void tst_generatedOutput::examplesManifestXmlAndQhp()
 {
     testAndCompare("testdata/configs/examples-qhp.qdocconf",
                    "examples-manifest.xml "
-                   "demos-manifest.xml "
+                   "test-demos-demo-example.html "
                    "test.qhp");
 }
 
@@ -333,6 +390,7 @@ void tst_generatedOutput::templateParameters()
     testAndCompare("testdata/configs/testtemplate.qdocconf",
                    "template/testqdoc-test.html "
                    "template/testqdoc-test-struct.html "
+                   "template/testqdoc-vec.html "
                    "template/foo.html "
                    "template/bar.html "
                    "template/baz.html");
@@ -342,6 +400,8 @@ void tst_generatedOutput::scopedEnum()
 {
     testAndCompare("testdata/configs/scopedenum.qdocconf",
                    "scopedenum/testqdoc-test.html "
+                   "scopedenum/scoped-enum-linking.html "
+                   "scopedenum-docbook/scoped-enum-linking.xml "
                    "scopedenum-docbook/testqdoc-test.xml");
 }
 
@@ -356,17 +416,37 @@ void tst_generatedOutput::dontDocument()
 void tst_generatedOutput::inheritedQmlPropertyGroups()
 {
     testAndCompare("testdata/qmlpropertygroups/qmlpropertygroups.qdocconf",
-                   "qmlpropertygroups/qml-qdoc-test-anotherchild-members.html");
+                   "qmlpropertygroups/qml-qdoc-test-anotherchild-members.html "
+                   "qmlpropertygroups/qml-qdoc-test-parent.html "
+                   "qmlpropertygroups-docbook/qml-qdoc-test-parent.xml");
+}
+
+void tst_generatedOutput::indexLinking()
+{
+    {
+        QScopedValueRollback<bool> skipRegen(m_regen, false);
+        inheritedQmlPropertyGroups();
+    }
+    copyIndexFiles();
+    QString indexDir = QLatin1String("-indexdir ") +  m_outputDir->path();
+    testAndCompare("testdata/indexlinking/indexlinking.qdocconf",
+                   "index-linking.html "
+                   "qml-linkmodule-grandchild-members.html",
+                   indexDir.toLatin1().data());
 }
 
 void tst_generatedOutput::crossModuleLinking()
 {
-    htmlFromCpp();
+    {
+        QScopedValueRollback<bool> skipRegen(m_regen, false);
+        htmlFromCpp();
+    }
     copyIndexFiles();
     QString indexDir = QLatin1String("-indexdir ") +  m_outputDir->path();
     testAndCompare("testdata/crossmodule/crossmodule.qdocconf",
                    "crossmodule/testtype.html "
-                   "crossmodule/testtype-members.html",
+                   "crossmodule/testtype-members.html "
+                   "crossmodule/crossmoduleref-sub-crossmodule.html",
                    indexDir.toLatin1().data());
 }
 
@@ -386,6 +466,8 @@ void tst_generatedOutput::singleExec()
                    "testqdoc-test.html "
                    "testqdoc-test-members.html "
                    "testqdoc.html "
+                   "crossmoduleref.html "
+                   "crossmodule/all-namespaces.html "
                    "crossmodule/testtype.html "
                    "crossmodule/testtype-members.html",
                    "-single-exec");
@@ -422,7 +504,8 @@ void tst_generatedOutput::noAutoList()
 void tst_generatedOutput::nestedMacro()
 {
     testAndCompare("testdata/configs/nestedmacro.qdocconf",
-                   "nestedmacro/testcpp-module.html");
+                   "nestedmacro/testcpp-module.html "
+                   "docbook-nestedmacro/testcpp-module.xml");
 }
 
 void tst_generatedOutput::headerFile()
@@ -434,6 +517,43 @@ void tst_generatedOutput::headerFile()
                    "headerfile-docbook/headers.xml");
 }
 
+void tst_generatedOutput::usingDirective()
+{
+    testAndCompare("testdata/configs/usingdirective.qdocconf", "space.html");
+}
+
+void tst_generatedOutput::properties()
+{
+    if (m_extraParams.isEmpty() && !m_regen) {
+        QSKIP("Required include paths not available");
+    }
+
+    testAndCompare("testdata/configs/properties.qdocconf",
+                   "properties/testqdoc-testderived.html "
+                   "properties/testqdoc-testderived-members.html "
+                   "properties/qml-thetype.html "
+                   "properties/testcpp.index "
+                   "properties-docbook/testqdoc-testderived.xml",
+                   m_extraParams.toLatin1().data());
+}
+
+void tst_generatedOutput::testTagFile()
+{
+    testAndCompare("testdata/configs/tagfiles.qdocconf", "testtagfile.tags");
+}
+
+void tst_generatedOutput::testGlobalFunctions()
+{
+    testAndCompare("testdata/configs/testglobals.qdocconf", "globals.html");
+}
+
+void tst_generatedOutput::proxyPage()
+{
+    testAndCompare("testdata/proxypage/proxypage.qdocconf",
+                   "proxypage/stdpair-proxypage-proxy.html "
+                   "proxypage-docbook/stdpair-proxypage-proxy.xml");
+}
+
 int main(int argc, char *argv[])
 {
     tst_generatedOutput tc;
